using System;
using System.Data;
using System.Windows.Forms;

namespace x_y_drag
{
	/// <summary>
	/// Summary description for Class1.
	/// </summary>
	public class Ball
	{
		public int balls = 0;

		private double mass, radius;
		private double xAngVelocity;
		private double yAngVelocity;
		private double zAngVelocity;
		private double xVelocity, yVelocity, zVelocity;
		private double _x, _y, _z, _xOld, _yOld, _zOld;
		private double c_d, c_m; //drag, magnus coefficient

		/// <summary>
		/// Constants
		/// </summary>
		private double gravity = -9.815; //ms^-2
		private double airDensity = 1.293; //kg/m^3
		private double airViscosity = 17.4E-6; //Pa*s

		/// <summary>
		/// The different properties of the ball class
		/// </summary>
		public double Mass
		{
			set { mass = value; }
			get { return mass; }
		}
		public double Radius
		{
			set { radius = value; }
			get { return radius; }
		}
		public double XAngVelocity //in rad/s
		{
			set { xAngVelocity = value; }
			get { return xAngVelocity; } //must be updated each loop
		}
		public double YAngVelocity //in rad/s
		{
			set { yAngVelocity = value; }
			get { return yAngVelocity; } //must be updated each loop
		}
		public double ZAngVelocity //in rad/s
		{
			set { zAngVelocity = value; }
			get { return zAngVelocity; } //must be updated each loop
		}

		public double XVelocity
		{
			set { xVelocity = value; }
			get { return xVelocity; }
		}
		public double YVelocity
		{
			set { yVelocity = value; }
			get { return yVelocity; }
		}
		public double ZVelocity
		{
			set { zVelocity = value; }
			get { return zVelocity; }
		}
		public double _X
		{
			set { _x = value; }
			get { return _x; }
		}
		public double _Y
		{
			set { _y = value; }
			get { return _y; }
		}
		public double _Z
		{
			set { _z = value; }
			get { return _z; }
		}
		public double C_d
		{
			set { c_d = value; }
			get { return c_d; }
		}
		public double C_m
		{
			set { c_m = value; }
			get { return c_m; }
		}
		public Ball()
		{
			this.C_d = 0.5;
			this.C_m = 1; //magnus coefficient
			balls++;
		}
		public Ball(double initV, double initAng)
		{		
			this.XVelocity = initV*Math.Cos(initAng);
			this.YVelocity = initV*Math.Sin(initAng);
			this.ZVelocity = 0;
			this.C_d = 0.5;
			this.C_m = 1; //magnus coefficient
			this.XAngVelocity = 0;
			this.YAngVelocity = 0;
			this.ZAngVelocity = 0;
			balls++;
		}

		#region Region: Here the different formulas are - GET
		public double Area()
		{
			return Math.Round(this.Radius*this.Radius*Math.PI, 4);
		}
		public double GetDrag() //Returns F_d
		{
			return -0.5 * this.C_d * this.airDensity * Area()* this.GetVelocity();
		}

		public double GetVelocity()
		{
			return Math.Sqrt(	this.XVelocity*this.XVelocity + 
								this.YVelocity*this.YVelocity + 
								this.ZVelocity*this.ZVelocity);
		}
		public double GetRenolds()
		{
			return (airDensity*this.GetVelocity()*this.Radius*2)/airViscosity;
		}
		public double GetRenolds(double v) //Returns constant depending on v
		{
			return (airDensity*v*this.Radius*2)/airViscosity;
		}
		public double GetMagnusApprox() //returns F_m
		{
			return (Math.PI*this.airDensity*this.GetVelocity()*this.Radius*this.Radius*this.Radius*this.Radius*this.ZAngVelocity)/(2*this.Radius);
		}

		//TODO: function to get C_d constant in order to change drag depending on speed/type of ball...
		/*
		private double getCd()
		{
			//experimental data needed to show correlation!
		}		
		*/
		#endregion
	}
}
